/***************************************************************************                          
    file:           sensordev.cpp
    description:    sensor device
    begin:          09/06/2024
    Copyright:      Giuseppe Della Bianca<giusdbg@gmail.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qcoreevent.h>
#include <qdebug.h>

#include "general.h"
#include "sensordevslist.h"
#include "sensordev.h"


SensorDev::SensorDev( SensorDevsList *sensorDevsListIn, const QString &name ) : QObject( /* parent */ /* ,name */ )
{
    sensorDevsList = sensorDevsListIn;
    
   setObjectName ( name );
   
 tempScale= Sensor::dgCelsius;

 monitorized = false;
 
    idTimer = 0;
    
    confUpdateInterval = 0;
    sensorUpdateInterval = 5000;    
    timerUpdateInterval = sensorUpdateInterval;
    withGlobalTimerUpdate = false;
 
    setParent ( sensorDevsListIn ); 
}

SensorDev::~SensorDev()
{
}


void SensorDev::setMonitorized(bool enable)
{
 if(monitorized!=enable) {
   monitorized= enable;
   if(enable)
   {
/*      if ( !idTimer )
         idTimer = startTimer( updateInterval ); */
   }
   else
   {
      if ( idTimer )
      {
         killTimer ( idTimer );
         idTimer = 0;
      }
   }
 }
}

bool SensorDev::getMonitorized()
{
    return monitorized;
}

void SensorDev::setTempScale(Sensor::TempScale scale)
{
 if(tempScale!=scale) {
     tempScale= scale;
 }
}

/***************************************************************************
    set and get of the configuration and sensor update interval
 ***************************************************************************/

void SensorDev::setConfUpdateInterval( double seconds )
{
    confUpdateInterval = ( seconds * 1000 );
}

double SensorDev::getConfUpdateInterval ( bool enabSec )
{
    return ( confUpdateInterval / ( enabSec ? 1000.0 : 1.0 ) );
}

void SensorDev::setSensorUpdateInterval( double seconds )
{
    int updateIntervalNew;

    updateIntervalNew = ( seconds * 1000 );

    if( sensorUpdateInterval != updateIntervalNew )
    {
        sensorUpdateInterval = updateIntervalNew;
        timerUpdateInterval = sensorUpdateInterval;
    
    if(monitorized) {
       setMonitorized(false);
       setMonitorized(true);
    }
    }
}

double SensorDev::getSensorUpdateInterval( bool enabSec )
{
    return ( sensorUpdateInterval / ( enabSec ? 1000.0 : 1.0 ) );
}

/***************************************************************************
    get update interval
 ***************************************************************************/
// warning, when called from the configuration window it should not take into account the configuration values
void SensorDev::getUpdateInterval ( double &confUpdateInterval, double &sensorsUpdateInterval )
{
    bool enabDisk;
    double confUpdateIntervalNew;

    confUpdateIntervalNew = confUpdateInterval;

//  setGroupConf ( objectName() );
    setSensorDevGroupConf ( getShortName(), getLongName(), getSubName() );

    if ( doubleCompare ( confUpdateInterval, -1.0 ) <= 0 )
        confUpdateIntervalNew = getSettingUpdateInterval ( qsFL1( "UpdateInterval" ), -1.0, Type::Default );
    if ( doubleCompare ( sensorsUpdateInterval, -1.0 ) <= 0 )
    {
        if ( qFuzzyIsNull ( confUpdateInterval ) )
            sensorsUpdateInterval = getSettingUpdateInterval ( qsFL1( "UpdateInterval" ), -1.0, Type::OnlyDefault );
        else
            sensorsUpdateInterval = getSettingUpdateInterval ( qsFL1( "UpdateInterval" ), -1.0, Type::AutoDefault );
    }

    enabDisk = false;

    if ( !enabDisk )
        enabDisk = sensorClass == Sensor::SensorClass::hdSensor;

    if ( !enabDisk && sensorClass == Sensor::SensorClass::lmSensor )
        enabDisk = getLongName().indexOf ( qsFL1( "drive" ) ) >= 0 || getLongName().indexOf ( qsFL1( "nvme" ) ) >= 0;

    if ( enabDisk )
    {
        if ( qFuzzyIsNull ( confUpdateIntervalNew ) && doubleCompare ( sensorsUpdateInterval, 3.0 ) < 0 )
            sensorsUpdateInterval = 3.0;
    }

    confUpdateInterval = confUpdateIntervalNew;
}

void SensorDev::readConfig()
{
    double confUpdateInterval, sensorsUpdateInterval;

    QString str;

    confUpdateInterval = sensorsUpdateInterval = -1.0;

    getUpdateInterval ( confUpdateInterval, sensorsUpdateInterval );

    setConfUpdateInterval ( confUpdateInterval );
    setSensorUpdateInterval ( sensorsUpdateInterval );


    str = getEntryConf ( qsFL1( "Scale" ), QString(), ksensorsSetting::AutoDefault );
 
 Sensor::TempScale tempScale;
 if(str== qsFL1( "F" ) ) tempScale= Sensor::dgFahrenheit;
 else if(str== qsFL1( "K" ) ) tempScale= Sensor::dgKelvin;
      else             tempScale= Sensor::dgCelsius;
 setTempScale(tempScale);
}

void SensorDev::writeConfig()
{
//  setGroupConf ( objectName() );
    setSensorDevGroupConf ( getShortName(), getLongName(), getSubName() );
    setEntryConf ( qsFL1( "ShortName" ),  getShortName() );
    setEntryConf ( qsFL1( "SubName" ),  getSubName() );    
    setEntryConf ( qsFL1( "UpdateInterval" ), ( confUpdateInterval / 1000.0 ), 0, ksensorsSetting::DeleteDefault );

    if ( getClass() != Sensor::sysSensor )
    {
   
 QString str;
 switch(tempScale) {
  case Sensor::dgCelsius   : str = QChar::fromLatin1( 'C' ); break;
  case Sensor::dgKelvin    : str = QChar::fromLatin1( 'K' ); break;
  case Sensor::dgFahrenheit: str = QChar::fromLatin1( 'F' ); break;
  default:
 
   break;
 }
      setEntryConf ( qsFL1( "Scale" ), str);
 
    }
}

/***************************************************************************
    get/set class, names e description, object name of sensor device
 ***************************************************************************/

void SensorDev::setClass( Sensor::SensorClass newClass )
{
    sensorClass = newClass;
}

Sensor::SensorClass SensorDev::getClass()
{
    return sensorClass;
}

void SensorDev::setShortName( const QString &name )
{
    shortName = name;
}

QString SensorDev::getShortName()
{
    return shortName;
}
    
void SensorDev::setLongName( const QString &name )
{
    longName = name;
}

QString SensorDev::getLongName()
{
    return longName;
}

void SensorDev::setSubName( const QString &name )
{
    subName = name;
}

QString SensorDev::getSubName()
{
    return subName;
}

void SensorDev::setDescription( const QString &desc )
{
    description = desc;
}

const QString &SensorDev::getDescription()
{
    return description;
}

void SensorDev::setObjectName( const QString &name )
{
    QObject::setObjectName( name );
    
    setShortName( name );
    setLongName( name );
}

/***************************************************************************
    global timer update
 ***************************************************************************/

void SensorDev::globalTimerUpdate ( int updateIntervalIn, int updateIntervalTimerIn )
{
    int oldTimerInterval;

    Q_UNUSED ( updateIntervalIn );
    
//    qDebug() << "\033[91m" << "qDebug(): SensorDev::globalTimerUpdate()" << objectName() << updateIntervalIn << updateIntervalTimerIn <<  "\033[0m";

    if ( !withGlobalTimerUpdate && timerUpdateInterval > 0 )
    {
//        qDebug() << "\033[91m" << "qDebug(): SensorDev::globalTimerUpdate()" << "start" << timerUpdateInterval << "\033[0m";

        withGlobalTimerUpdate = true;

        oldTimerInterval = timerUpdateInterval;

        timerUpdateInterval -= updateIntervalTimerIn;

        if ( oldTimerInterval > 0 && timerUpdateInterval <= 0 )
        {
//            qDebug() << "\033[91m" << "qDebug(): SensorDev::globalTimerUpdate()" << "updateSensors start" << withGlobalTimerUpdate <<  "\033[0m";

            updateSensors ( false );

            timerUpdateInterval = sensorUpdateInterval;

//            qDebug() << "\033[91m" << "qDebug(): SensorDev::globalTimerUpdate()" << "updateSensors end" << withGlobalTimerUpdate << "\033[0m";
        }
        else
            setWithGlobalTimerUpdate( false );
      
//        qDebug() << "\033[91m" << "qDebug(): SensorDev::globalTimerUpdate()" << "end" << timerUpdateInterval << "\033[0m";

    } // if ( !withGlobalTimerUpdate && timerUpdateInterval > 0 )
} // void globalTimerUpdate ( int updateIntervalIn, int updateIntervalTimerIn )

/***************************************************************************
    set global timer update active state
 ***************************************************************************/

void SensorDev::setWithGlobalTimerUpdate ( bool withGlobalTimerUpdateIn )
{
    withGlobalTimerUpdate = withGlobalTimerUpdateIn;
}

//****************************************************************************
// Protected methods
//****************************************************************************

void SensorDev::timerEvent( QTimerEvent * )
{
//   qDebug() << "\033[91m" << "qDebug(): SensorDev::timerEvent" << idTimer << "\033[0m";
   
    updateSensors ( false );
 
//   qDebug() << "\033[91m" << "qDebug(): SensorDev::timerEvent" << idTimer << "post" << "\033[0m";
}

//***************

void SensorDev::childEvent ( QChildEvent *e )
{   
//   qDebug() << "\033[91m" << "qDebug(): SensorDev::childEvent()" << "\033[0m";
   
 if(e->added()) {
    
//      qDebug() << "\033[91m" << "qDebug(): SensorDev::childEvent()" << "e->added()" <<"\033[0m";

    connect((Sensor *)e->child(),SIGNAL(valueChanged()) ,this,SLOT(slotValueChanged()));
 }
}

//****************************************************************************
// Protected Q_SLOTS
//****************************************************************************

void SensorDev::slotValueChanged()
{
    Q_EMIT valueChanged( (Sensor *)sender() );
}


/*********************************************************************************/
